package rbccm.felix.objectdb.test;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import javax.persistence.Entity;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.LockModeType;
import javax.persistence.Persistence;

import junit.framework.Assert;

import org.junit.Test;

public class TestObjectUpdating {
	
	public static void main(String[] args) {	
		TestObjectUpdating test = new TestObjectUpdating();
		test.testObjectMultiThreadedUpdate();
	}

	@Test
	public void testObjectMultiThreadedUpdate() {
		System.out.println("Starting...");
		EntityManagerFactory emf = Persistence.createEntityManagerFactory("$objectdb/db/testObjectMultiThreadedUpdate.odb");
		try {

			MyEntity entity = new MyEntity("start");
			
			EntityManager em = emf.createEntityManager();
			em.getTransaction().begin();
			em.persist(entity);
	        em.getTransaction().commit();
			
			ArrayList<Thread> runnables = new ArrayList<Thread>();
			for(int i=0; i<5; i++)
				runnables.add(new ObjectDbUpdater(emf, "updater" + i));
			
			long start = System.currentTimeMillis();
			
			for(int i=0; i<5; i++)
				runnables.get(i).start();
			
			for(int i=0; i<5; i++)
				runnables.get(i).join();
			
			long runTime = System.currentTimeMillis() - start;
			System.out.println(String.format("Total elapsed test time %sms", runTime));
			
			System.out.println("done");
		} catch (Throwable e) {
			e.printStackTrace();
			Assert.fail();
		} finally {
			emf.close();
		}
	}
	
	private class ObjectDbUpdater extends Thread {
		private EntityManagerFactory _emf;
		private String _name;
		
		public ObjectDbUpdater(EntityManagerFactory emf, String name) {
			_emf = emf;
			_name = name;
		}
		
		@Override
		public void run() {
			EntityManager em = _emf.createEntityManager();
			try {
				System.out.println(String.format("%s starting", _name));
				
				em.getTransaction().begin();
				
				Map<String,Object> properties = new HashMap<String,Object>();
				properties.put("javax.persistence.lock.timeout", 1000);
				
				MyEntity loaded = em.find(MyEntity.class, 1, LockModeType.PESSIMISTIC_WRITE, properties);
				
				System.out.println(String.format("%s changing name from %s to %s", _name, loaded.getName(), _name));
				loaded.setName(_name);
				
				em.persist(loaded);
				em.getTransaction().commit();
				
				System.out.println(String.format("%s commited", _name));
			} catch(Exception e) {
				e.printStackTrace();
				
				if(em.getTransaction().isActive())
					em.getTransaction().rollback();
			} finally {
				em.close();
			}
		}
	}
	
	@Entity
    public static final class MyEntity {
        private String name;
        public MyEntity(String name) {
            this.name = name;
        }
        
        public String getName() { return name; }
        public void setName(String value) { name = value; }
    }
}
